"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDevGatewayServer = void 0;
const apollo_server_1 = require("apollo-server");
const apollo_server_core_1 = require("apollo-server-core");
const gateway_1 = require("@apollo/gateway");
const legacyLoadServicesFromStorage_1 = require("@apollo/gateway/dist/legacyLoadServicesFromStorage");
const loadServicesFromRemoteEndpoint_1 = require("@apollo/gateway/dist/loadServicesFromRemoteEndpoint");
const RemoteGraphQLDataSource_1 = require("./datasources/RemoteGraphQLDataSource");
const utils_1 = require("./utils");
function createDevGatewayServer(url, port, serviceName, key, studioUrl, gatewayUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        const localServiceList = yield getLocalServiceList(url);
        let studioServiceList = [];
        if (serviceName && key) {
            studioServiceList = yield getStudioServiceList(serviceName, key, gatewayUrl);
        }
        const gateway = new gateway_1.ApolloGateway({
            debug: true,
            localServiceList: [...localServiceList, ...studioServiceList],
            buildService({ url }) {
                return new RemoteGraphQLDataSource_1.RemoteGraphlQLDataSource({ url });
            }
        });
        const server = new apollo_server_1.ApolloServer({
            apollo: { key },
            gateway,
            subscriptions: false,
            context({ req, res }) {
                return { req, res };
            },
            playground: true,
            plugins: key ? [
                apollo_server_core_1.ApolloServerPluginUsageReporting({
                    endpointUrl: studioUrl,
                    sendHeaders: {
                        all: true
                    },
                    sendVariableValues: {
                        all: true,
                    },
                }),
            ] : undefined,
            cors: {
                origin(origin, callback) {
                    // callback(null, !!origin && /\.ksyun\.com(:\d+)?$/.test(origin));
                    callback(null, true);
                },
                credentials: true,
            }
        });
        server.listen(port).then(({ url }) => {
            console.log(`🚀 Gateway server ready at ${url}`);
        });
    });
}
exports.createDevGatewayServer = createDevGatewayServer;
function getStudioServiceList(serviceName, key, gatewayUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        const fetcher = gateway_1.getDefaultGcsFetcher();
        const apiKeyHash = utils_1.createSHA('sha512').update(key).digest('hex');
        const graphId = key.split(':', 2)[1];
        const { serviceDefinitions } = yield legacyLoadServicesFromStorage_1.getServiceDefinitionsFromStorage({
            graphId,
            apiKeyHash,
            graphVariant: 'current',
            federationVersion: 1,
            fetcher,
        });
        const serviceList = [];
        if (serviceDefinitions) {
            serviceDefinitions.map(({ name, url, typeDefs }) => {
                if (name === serviceName)
                    return;
                serviceList.push({
                    name,
                    url: gatewayUrl + '/' + graphId,
                    typeDefs
                });
            });
        }
        return serviceList;
    });
}
function getLocalServiceList(url) {
    return __awaiter(this, void 0, void 0, function* () {
        const dataSource = new gateway_1.RemoteGraphQLDataSource({ url });
        const serviceList = [{ name: '__local__', url, dataSource }];
        const { serviceDefinitions } = yield loadServicesFromRemoteEndpoint_1.getServiceDefinitionsFromRemoteEndpoint({
            serviceList,
            getServiceIntrospectionHeaders: () => __awaiter(this, void 0, void 0, function* () { return undefined; }),
            serviceSdlCache: new Map(),
        });
        return serviceDefinitions || [];
    });
}
